/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "arvoconfig_key_combo_box.h"
#include "roccat_warning_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_timer_dialog.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define ARVOCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), ARVOCONFIG_KEY_COMBO_BOX_TYPE, ArvoconfigKeyComboBoxClass))
#define IS_ARVOCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), ARVOCONFIG_KEY_COMBO_BOX_TYPE))
#define ARVOCONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), ARVOCONFIG_KEY_COMBO_BOX_TYPE, ArvoconfigKeyComboBoxClass))
#define ARVOCONFIG_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), ARVOCONFIG_KEY_COMBO_BOX_TYPE, ArvoconfigKeyComboBoxPrivate))

typedef struct _ArvoconfigKeyComboBoxClass ArvoconfigKeyComboBoxClass;
typedef struct _ArvoconfigKeyComboBoxPrivate ArvoconfigKeyComboBoxPrivate;

struct _ArvoconfigKeyComboBox {
	GtkComboBox parent;
	ArvoconfigKeyComboBoxPrivate *priv;
};

struct _ArvoconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _ArvoconfigKeyComboBoxPrivate {
	ArvoRkpButtonInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(ArvoconfigKeyComboBox, arvoconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(ArvoconfigKeyComboBox *key_combo_box) {
	ArvoconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(ArvoRkpButtonInfo const *button_info) {
	return roccat_shortcut_dialog_create_name(button_info->shortcut_ctrl == ARVO_RKP_BUTTON_INFO_SHORTCUT_STATE_ON,
			button_info->shortcut_shift == ARVO_RKP_BUTTON_INFO_SHORTCUT_STATE_ON,
			button_info->shortcut_alt == ARVO_RKP_BUTTON_INFO_SHORTCUT_STATE_ON,
			button_info->shortcut_win == ARVO_RKP_BUTTON_INFO_SHORTCUT_STATE_ON,
			button_info->shortcut_key);
}

static void set_label(ArvoconfigKeyComboBox *key_combo_box) {
	ArvoconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext;

	switch (priv->key_info->type) {
	case ARVO_RKP_BUTTON_INFO_TYPE_SHORTCUT:
		labeltext = shortcut_name(priv->key_info);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar const *)priv->key_info->macroset_name, (gchar const *)priv->key_info->macro_name);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_QUICKLAUNCH_NS:
		labeltext = g_path_get_basename((gchar const *)priv->key_info->macroset_name);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_TIMER_START:
		labeltext = g_strdup_printf("%s", (gchar const *)priv->key_info->timer_name);
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static ArvoRkpButtonInfo *button_quicklaunch_ns(GtkWindow *parent) {
	GtkWidget *dialog;
	gchar *filename;
	ArvoRkpButtonInfo *key_info = NULL;

	dialog = gtk_file_chooser_dialog_new("Select executable",
			parent,
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL
	);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		key_info = arvo_rkp_button_info_new_quicklaunch_ns(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
	return key_info;
}

static ArvoRkpButtonInfo *button_macro(GtkWindow *parent) {
	ArvoRkpButtonInfo *key_info = NULL;
	RoccatMacro *roccat_macro;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		key_info = roccat_macro_to_arvo_rkp_button_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static ArvoRkpButtonInfo *button_shortcut(GtkWindow *parent) {
	RoccatShortcutDialog *dialog;
	ArvoRkpButtonInfo *key_info = NULL;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = arvo_rkp_button_info_new_shortcut(
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				roccat_shortcut_dialog_get_shift(dialog),
				roccat_shortcut_dialog_get_ctrl(dialog),
				roccat_shortcut_dialog_get_win(dialog),
				roccat_shortcut_dialog_get_alt(dialog)
		);
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static ArvoRkpButtonInfo *button_timer(GtkWindow *parent) {
	ArvoRkpButtonInfo *key_info = NULL;
	RoccatTimer *timer;

	timer = roccat_timer_dialog(parent);

	if (timer) {
		key_info = arvo_rkp_button_info_new_timer(timer->name, roccat_timer_get_seconds(timer));
		roccat_timer_free(timer);
	}

	return key_info;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	ArvoconfigKeyComboBox *key_combo_box = ARVOCONFIG_KEY_COMBO_BOX(combo);
	ArvoconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	GtkTreeIter iter;
	gboolean valid;
	gint type_data;
	ArvoRkpButtonInfo *new_key_info;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type_data, -1);

	switch (type_data) {
	case ARVO_RKP_BUTTON_INFO_TYPE_MACRO:
		new_key_info = button_macro(parent);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_SHORTCUT:
		new_key_info = button_shortcut(parent);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_QUICKLAUNCH_NS:
		new_key_info = button_quicklaunch_ns(parent);
		break;
	case ARVO_RKP_BUTTON_INFO_TYPE_TIMER_START:
		new_key_info = button_timer(parent);
		break;
	default:
		new_key_info = arvo_rkp_button_info_new_special(type_data);
	}

	if (new_key_info) {
		g_free(priv->key_info);
		priv->key_info = new_key_info;
	} else
		set_active_iter(key_combo_box);

	set_label(key_combo_box);
}

GtkWidget *arvoconfig_key_combo_box_new(GtkLabel *label) {
	ArvoconfigKeyComboBox *key_combo_box;
	ArvoconfigKeyComboBoxClass *key_combo_box_class;
	ArvoconfigKeyComboBoxPrivate *priv;

	key_combo_box = ARVOCONFIG_KEY_COMBO_BOX(g_object_new(ARVOCONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = ARVOCONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), key_combo_box_class->model);

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void arvoconfig_key_combo_box_init(ArvoconfigKeyComboBox *key_combo_box) {
	ArvoconfigKeyComboBoxPrivate *priv = ARVOCONFIG_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel, sublevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "WWW", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Search", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_SEARCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Home", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_HOME, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Back", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_BACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Forward", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Stop", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Refresh", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_REFRESH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "WWW Favorites", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_WWW_FAVORITES, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "*Profile", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "*Profile cycle", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_PROFILE_CYCLE_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "*Profile up", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_PROFILE_UP_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "*Profile down", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_PROFILE_DOWN_NS, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "*Easyshift", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, &sublevel, &toplevel, TITLE_COLUMN, "*Other", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyshift", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYSHIFT_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyshift lock", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYSHIFT_LOCK_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyaim Setting 1", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYAIM_1_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyaim Setting 2", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYAIM_2_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyaim Setting 3", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYAIM_3_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyaim Setting 4", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYAIM_4_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "*Other Easyaim Setting 5", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TALK_OTHER_EASYAIM_5_NS, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_MACRO, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_OFF, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Timer", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TIMER_START, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Timer stop", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_TIMER_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "*Open driver", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_OPEN_DRIVER_NS, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "*Quicklaunch", TYPE_COLUMN, ARVO_RKP_BUTTON_INFO_TYPE_QUICKLAUNCH_NS, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	ArvoconfigKeyComboBoxPrivate *priv = ARVOCONFIG_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(arvoconfig_key_combo_box_parent_class)->finalize(object);
	g_free(priv->key_info);
}

static void arvoconfig_key_combo_box_class_init(ArvoconfigKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(ArvoconfigKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void arvoconfig_key_combo_box_set_value_blocked(ArvoconfigKeyComboBox *key_combo_box, ArvoRkpButtonInfo const *key_info) {
	ArvoconfigKeyComboBoxPrivate *priv = key_combo_box->priv;

	g_free(priv->key_info);
	priv->key_info = arvo_rkp_button_info_dup(key_info);
	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

ArvoRkpButtonInfo *arvoconfig_key_combo_box_get_value(ArvoconfigKeyComboBox *key_combo_box) {
	return arvo_rkp_button_info_dup(key_combo_box->priv->key_info);
}
