/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "isku_key_combo_box.h"
#include "roccat_talk_target_dialog.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_timer_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define ISKU_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), ISKU_KEY_COMBO_BOX_TYPE, IskuKeyComboBoxClass))
#define IS_ISKU_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), ISKU_KEY_COMBO_BOX_TYPE))
#define ISKU_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), ISKU_KEY_COMBO_BOX_TYPE, IskuKeyComboBoxClass))
#define ISKU_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), ISKU_KEY_COMBO_BOX_TYPE, IskuKeyComboBoxPrivate))

typedef struct _IskuKeyComboBoxClass IskuKeyComboBoxClass;
typedef struct _IskuKeyComboBoxPrivate IskuKeyComboBoxPrivate;

struct _IskuKeyComboBox {
	GtkComboBox parent;
	IskuKeyComboBoxPrivate *priv;
};

struct _IskuKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _IskuKeyComboBoxPrivate {
	gint standard;
	IskuRkpMacroKeyInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(IskuKeyComboBox, isku_key_combo_box, GTK_TYPE_COMBO_BOX);

static gboolean visible_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer pmask) {
	gint type;
	guint mask;

	mask = GPOINTER_TO_UINT(pmask);

	gtk_tree_model_get(model, iter, TYPE_COLUMN, &type, -1);
	if (mask & ISKU_KEY_COMBO_BOX_NO_MACRO && type == ISKU_KEY_TYPE_MACRO)
		return FALSE;
	if (mask & ISKU_KEY_COMBO_BOX_NO_SHORTCUT && type == ISKU_KEY_TYPE_SHORTCUT)
		return FALSE;

	return TRUE;
}

static GtkTreeModel *filter(GtkTreeModel *child, guint mask) {
	GtkTreeModel *filter;

	filter = gtk_tree_model_filter_new(child, NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filter), visible_func, GUINT_TO_POINTER(mask), NULL);
	return filter;
}

static void set_active_iter(IskuKeyComboBox *key_combo_box) {
	IskuKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(IskuRkpMacroKeyInfo const *key_info) {
	return roccat_shortcut_dialog_create_name(roccat_get_bit(key_info->keystrokes[0].action, ISKU_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_CTRL),
			roccat_get_bit(key_info->keystrokes[0].action, ISKU_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_SHIFT),
			roccat_get_bit(key_info->keystrokes[0].action, ISKU_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_ALT),
			roccat_get_bit(key_info->keystrokes[0].action, ISKU_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_WIN),
			key_info->keystrokes[0].key);
}

static void set_label(IskuKeyComboBox *key_combo_box) {
	IskuKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext;

	switch (priv->key_info->type) {
	case ISKU_KEY_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar const *)priv->key_info->macroset_name, (gchar const *)priv->key_info->macro_name);
		break;
	case ISKU_KEY_TYPE_SHORTCUT:
		labeltext = shortcut_name(priv->key_info);
		break;
	case ISKU_KEY_TYPE_QUICKLAUNCH:
		labeltext = g_path_get_basename((gchar const *)priv->key_info->filename);
		break;
	case ISKU_KEY_TYPE_TIMER:
		labeltext = g_strdup_printf("%s", (gchar const *)priv->key_info->timer_name);
		break;
	case ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT_LOCK:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_1:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_2:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_3:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_4:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_5:
		labeltext = g_strdup(roccat_talk_target_dialog_get_label(isku_rkp_macro_key_info_get_talk_device(priv->key_info)));
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static IskuRkpMacroKeyInfo *button_quicklaunch(GtkWindow *parent) {
	GtkWidget *dialog;
	gchar *filename;
	IskuRkpMacroKeyInfo *key_info = NULL;

	dialog = gtk_file_chooser_dialog_new("Select executable",
			parent,
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		key_info = isku_rkp_macro_key_info_new_quicklaunch(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
	return key_info;
}

static IskuRkpMacroKeyInfo *button_macro(GtkWindow *parent) {
	RoccatMacro *roccat_macro;
	IskuRkpMacroKeyInfo *key_info = NULL;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		key_info = roccat_macro_to_isku_rkp_macro_key_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static IskuRkpMacroKeyInfo *button_shortcut(GtkWindow *parent) {
	RoccatShortcutDialog *dialog;
	IskuRkpMacroKeyInfo *key_info = NULL;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = isku_rkp_macro_key_info_new_shortcut(
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				isku_rkp_macro_key_info_build_shortcut_modifier(
						roccat_shortcut_dialog_get_ctrl(dialog),
						roccat_shortcut_dialog_get_shift(dialog),
						roccat_shortcut_dialog_get_alt(dialog),
						roccat_shortcut_dialog_get_win(dialog)));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static IskuRkpMacroKeyInfo *button_timer(GtkWindow *parent) {
	IskuRkpMacroKeyInfo *key_info = NULL;
	RoccatTimer *timer;

	timer = roccat_timer_dialog(parent);
	if (timer) {
		key_info = isku_rkp_macro_key_info_new_timer(timer->name, roccat_timer_get_seconds(timer));
		roccat_timer_free(timer);
	}
	return key_info;
}

static IskuRkpMacroKeyInfo *button_talk_other(GtkWindow *parent, gint key_type) {
	guint target = roccat_talk_target_dialog(parent, ROCCAT_TALK_DEVICE_ISKU);
	return isku_rkp_macro_key_info_new_talk(key_type, target);
}

static IskuRkpMacroKeyInfo *get_rkp_macro_key_info(GtkWindow *parent, gint type) {
	IskuRkpMacroKeyInfo *key_info;
	switch (type) {
	case ISKU_KEY_TYPE_MACRO:
		key_info = button_macro(parent);
		break;
	case ISKU_KEY_TYPE_SHORTCUT:
		key_info = button_shortcut(parent);
		break;
	case ISKU_KEY_TYPE_QUICKLAUNCH:
		key_info = button_quicklaunch(parent);
		break;
	case ISKU_KEY_TYPE_TIMER:
		key_info = button_timer(parent);
		break;
	case ISKU_KEY_TYPE_TALK_BOTH_EASYSHIFT:
		key_info = isku_rkp_macro_key_info_new_talk(type, ROCCAT_TALK_DEVICE_ALL);
		break;
	case ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT_LOCK:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_1:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_2:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_3:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_4:
	case ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_5:
		key_info = button_talk_other(parent, type);
		break;
	default:
		key_info = isku_rkp_macro_key_info_new_special(type);
	}
	return key_info;
}

static gint key_combo_box_get_active_type(GtkComboBox *combo) {
	GtkTreeIter iter;
	gint type;
	gboolean valid;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return -1;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type, -1);
	return type;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	IskuKeyComboBox *key_combo_box = ISKU_KEY_COMBO_BOX(combo);
	IskuKeyComboBoxPrivate *priv = key_combo_box->priv;
	IskuRkpMacroKeyInfo *new_key_info;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	gint actual_type, new_type = -1;

	actual_type = key_combo_box_get_active_type(combo);

	new_type = (actual_type == ISKU_KEY_TYPE_STANDARD) ? priv->standard : actual_type;

	new_key_info = get_rkp_macro_key_info(parent, new_type);

	if (new_key_info) {
		isku_rkp_macro_key_info_free(priv->key_info);
		priv->key_info = new_key_info;
		if (actual_type != new_type)
			set_active_iter(key_combo_box);
	} else
		set_active_iter(key_combo_box); /* set old iter active */

	set_label(key_combo_box);
}

GtkWidget *isku_key_combo_box_new(GtkLabel *label, guint mask, gint standard) {
	IskuKeyComboBox *key_combo_box;
	IskuKeyComboBoxClass *key_combo_box_class;
	IskuKeyComboBoxPrivate *priv;

	key_combo_box = ISKU_KEY_COMBO_BOX(g_object_new(ISKU_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;
	priv->standard = standard;

	key_combo_box_class = ISKU_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), filter(key_combo_box_class->model, mask));

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void isku_key_combo_box_init(IskuKeyComboBox *key_combo_box) {
	IskuKeyComboBoxPrivate *priv = ISKU_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static void key_store_add_standard(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Standards",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "A",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_A, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "B",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_B, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "C",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_C, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "D",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_D, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "E",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_E, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "G",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_G, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Q",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_Q, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "R",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_R, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "S",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_S, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "T",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_T, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "V",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_V, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "W",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_W, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "X",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_X, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Y",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_Y, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "1",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "2",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "3",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "4",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "5",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_5, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Capslock",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CAPSLOCK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F1",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F2",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F3",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F4",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F5",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F5, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F6",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F6, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F7",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F7, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F8",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F8, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F9",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F9, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F10",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F10, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F11",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F11, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "F12",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_F12, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Mute",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Volume up",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Volume down",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_VOLUME_DOWN, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Play/Pause",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Previous track",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Next track",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Webbrowser",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_APP_IE_BROWSER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "My Computer",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_APP_MY_COMPUTER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shift+1",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_SHIFT_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shift+2",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_SHIFT_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shift+3",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_SHIFT_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shift+4",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_SHIFT_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shift+5",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_SHIFT_5, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Ctrl+1",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CTRL_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Ctrl+2",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CTRL_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Ctrl+3",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CTRL_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Ctrl+4",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CTRL_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Ctrl+5",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD_CTRL_5, -1);
}

static void key_store_add_profile(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Profile",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile up",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile down",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_DOWN, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile cycle",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile 1",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile 2",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile 3",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile 4",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Profile 5",
			TYPE_COLUMN, ISKU_KEY_TYPE_PROFILE_5, -1);
}

static void key_store_add_multimedia(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Multimedia",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Open player",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Previous track",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Next track",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Play/Pause",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Stop",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Mute",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Volume up",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Volume down",
			TYPE_COLUMN, ISKU_KEY_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);
}

static void key_store_add_system(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "System",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Shut down",
			TYPE_COLUMN, ISKU_KEY_TYPE_SYSTEM_SHUT_DOWN, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Sleep",
			TYPE_COLUMN, ISKU_KEY_TYPE_SYSTEM_SLEEP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Wake",
			TYPE_COLUMN, ISKU_KEY_TYPE_SYSTEM_WAKE, -1);
}

static void key_store_add_timer(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Timer",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Timer start",
			TYPE_COLUMN, ISKU_KEY_TYPE_TIMER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Timer stop",
			TYPE_COLUMN, ISKU_KEY_TYPE_TIMER_STOP, -1);
}

static void key_store_add_www(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "WWW",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Search",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_SEARCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Home",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_HOME, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Back",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_BACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Forward",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Stop",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Refresh",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_REFRESH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Favorites",
			TYPE_COLUMN, ISKU_KEY_TYPE_WWW_FAVORITES, -1);
}

static void key_store_add_apps(GtkTreeStore *store) {
	GtkTreeIter toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Applications",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Quicklaunch",
			TYPE_COLUMN, ISKU_KEY_TYPE_QUICKLAUNCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Open driver",
			TYPE_COLUMN, ISKU_KEY_TYPE_OPEN_DRIVER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "eMail",
			TYPE_COLUMN, ISKU_KEY_TYPE_APP_EMAIL, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Calculator",
			TYPE_COLUMN, ISKU_KEY_TYPE_APP_CALCULATOR, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "My Computer",
			TYPE_COLUMN, ISKU_KEY_TYPE_APP_MY_COMPUTER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel,
			TITLE_COLUMN, "Webbrowser",
			TYPE_COLUMN, ISKU_KEY_TYPE_APP_IE_BROWSER, -1);
}

static void key_store_add_easyshift(GtkTreeStore *store) {
	GtkTreeIter iter, toplevel;

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL,
			TITLE_COLUMN, "Easyshift",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, &iter, &toplevel,
			TITLE_COLUMN, "Self",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Self Easyshift",
			TYPE_COLUMN, ISKU_KEY_TYPE_EASYSHIFT, -1);

	gtk_roccat_tree_store_append_with_values(store, &iter, &toplevel,
			TITLE_COLUMN, "Other",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyshift",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyshift lock",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYSHIFT_LOCK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyaim Setting 1",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyaim Setting 2",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyaim Setting 3",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyaim Setting 4",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Other Easyaim Setting 5",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_OTHER_EASYAIM_5, -1);

	gtk_roccat_tree_store_append_with_values(store, &iter, &toplevel,
			TITLE_COLUMN, "Both",
			TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &iter,
			TITLE_COLUMN, "Both Easyshift",
			TYPE_COLUMN, ISKU_KEY_TYPE_TALK_BOTH_EASYSHIFT, -1);
}

static void key_store_add_macro(GtkTreeStore *store) {
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Shortcut",
			TYPE_COLUMN, ISKU_KEY_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Macro",
			TYPE_COLUMN, ISKU_KEY_TYPE_MACRO, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Macro play/pause",
			TYPE_COLUMN, ISKU_KEY_TYPE_MACRO_PLAY_PAUSE, -1);
}

static void key_store_add_misc(GtkTreeStore *store) {
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Disabled",
			TYPE_COLUMN, ISKU_KEY_TYPE_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Unused",
			TYPE_COLUMN, ISKU_KEY_TYPE_UNUSED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL,
			TITLE_COLUMN, "Standard",
			TYPE_COLUMN, ISKU_KEY_TYPE_STANDARD, -1);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	key_store_add_standard(store);
	key_store_add_profile(store);
	key_store_add_multimedia(store);
	key_store_add_system(store);
	key_store_add_timer(store);
	key_store_add_www(store);
	key_store_add_apps(store);
	key_store_add_easyshift(store);
	key_store_add_macro(store);
	key_store_add_misc(store);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	IskuKeyComboBoxPrivate *priv = ISKU_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(isku_key_combo_box_parent_class)->finalize(object);
	isku_rkp_macro_key_info_free(priv->key_info);
}

static void isku_key_combo_box_class_init(IskuKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(IskuKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void isku_key_combo_box_set_value_blocked(IskuKeyComboBox *key_combo_box, IskuRkpMacroKeyInfo const *key_info) {
	IskuKeyComboBoxPrivate *priv = key_combo_box->priv;

	isku_rkp_macro_key_info_free(priv->key_info);

	if (key_info->type == ISKU_KEY_TYPE_STANDARD)
		priv->key_info = isku_rkp_macro_key_info_new_special(priv->standard);
	else
		priv->key_info = isku_rkp_macro_key_info_dup(key_info);

	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

IskuRkpMacroKeyInfo *isku_key_combo_box_get_value(IskuKeyComboBox *key_combo_box) {
	return isku_rkp_macro_key_info_dup(key_combo_box->priv->key_info);
}
