/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "koneconfig_key_combo_box.h"
#include "koneconfig_shortcut_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_warning_dialog.h"
#include "roccat_key_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define KONECONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONECONFIG_KEY_COMBO_BOX_TYPE, KoneconfigKeyComboBoxClass))
#define IS_KONECONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONECONFIG_KEY_COMBO_BOX_TYPE))
#define KONECONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), KONECONFIG_KEY_COMBO_BOX_TYPE, KoneconfigKeyComboBoxClass))
#define KONECONFIG_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONECONFIG_KEY_COMBO_BOX_TYPE, KoneconfigKeyComboBoxPrivate))

typedef struct _KoneconfigKeyComboBoxClass KoneconfigKeyComboBoxClass;
typedef struct _KoneconfigKeyComboBoxPrivate KoneconfigKeyComboBoxPrivate;

struct _KoneconfigKeyComboBox {
	GtkComboBox parent;
	KoneconfigKeyComboBoxPrivate *priv;
};

struct _KoneconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _KoneconfigKeyComboBoxPrivate {
	KoneRMPButtonInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(KoneconfigKeyComboBox, koneconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(KoneconfigKeyComboBox *key_combo_box) {
	KoneconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_create_name(KoneRMPButtonInfo const *button_info) {
	gchar *pre;

	switch(button_info->keystrokes[0].action) {
	case KONE_KEYSTROKE_ACTION_NONE:
		pre = "";
		break;
	case KONE_KEYSTROKE_ACTION_SHIFT:
		pre = "Shift+";
		break;
	case KONE_KEYSTROKE_ACTION_CTRL:
		pre = "Ctrl+";
		break;
	case KONE_KEYSTROKE_ACTION_ALT:
		pre = "Alt+";
		break;
	case KONE_KEYSTROKE_ACTION_WIN:
		pre = "Win+";
		break;
	default:
		g_critical("unknown action %i", button_info->keystrokes[0].action);
		pre = "";
	}
	return g_strdup_printf("%s%s", pre, roccat_hid_to_keyname(button_info->keystrokes[0].key));
}

static void set_label(KoneconfigKeyComboBox *key_combo_box) {
	KoneconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext = NULL;

	switch (priv->key_info->type) {
	case KONE_BUTTON_INFO_TYPE_KEY:
		labeltext = roccat_hid_to_keyname(priv->key_info->keystrokes[0].key);
		break;
	case KONE_BUTTON_INFO_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar *)priv->key_info->macro_set_name, (gchar *)priv->key_info->macro_name);
		break;
	case KONE_BUTTON_INFO_TYPE_SHORTCUT:
		labeltext = shortcut_create_name(priv->key_info);
		break;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static KoneRMPButtonInfo *button_key(GtkWindow *parent) {
	RoccatKeyDialog *dialog;
	KoneRMPButtonInfo *key_info = NULL;

	dialog = ROCCAT_KEY_DIALOG(roccat_key_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
		key_info = kone_rmp_button_info_new_single_key(roccat_keycode_to_hid(roccat_key_dialog_get_keycode(dialog)));

	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static void macro_is_too_long_dialog(GtkWindow *parent) {
	roccat_warning_dialog(parent,
			"Macro is too long to fit in hardware.",
			"It will be executed by software.");
}

static void macro_has_loop_dialog(GtkWindow *parent) {
	roccat_warning_dialog(parent,
			"Macro is looped which is not supported by hardware.",
			"It will be executed by software.");
}

static KoneRMPButtonInfo *button_macro(GtkWindow *parent) {
	RoccatMacro *roccat_macro;
	KoneRMPButtonInfo *key_info = NULL;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		if (roccat_macro_keystrokes_get_count(&roccat_macro->keystrokes) > KONE_BUTTON_INFO_KEYSTROKES_NUM)
			macro_is_too_long_dialog(parent);
		else if (roccat_macro->keystrokes.loop > 1)
			macro_has_loop_dialog(parent);
		
		key_info = roccat_macro_to_kone_rmp_button_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static KoneRMPButtonInfo *button_shortcut(GtkWindow *parent) {
	KoneconfigShortcutDialog *dialog;
	KoneRMPButtonInfo *key_info = NULL;

	dialog = KONECONFIG_SHORTCUT_DIALOG(koneconfig_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = kone_rmp_button_info_new_shortcut(
				roccat_keycode_to_hid(koneconfig_shortcut_dialog_get_keycode(dialog)),
				koneconfig_shortcut_dialog_get_modifier(dialog));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	KoneconfigKeyComboBox *key_combo_box = KONECONFIG_KEY_COMBO_BOX(combo);
	KoneconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	GtkTreeIter iter;
	gboolean valid;
	guint type_data;
	KoneRMPButtonInfo *new_key_info;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type_data, -1);

	switch (type_data) {
	case KONE_BUTTON_INFO_TYPE_KEY:
		new_key_info = button_key(parent);
		break;
	case KONE_BUTTON_INFO_TYPE_MACRO:
		new_key_info = button_macro(parent);
		break;
	case KONE_BUTTON_INFO_TYPE_SHORTCUT:
		new_key_info = button_shortcut(parent);
		break;
	default:
		new_key_info = kone_rmp_button_info_new_special(type_data);
	}

	if (new_key_info) {
		g_free(priv->key_info);
		priv->key_info = new_key_info;
	} else
		set_active_iter(key_combo_box);

	set_label(key_combo_box);
}

GtkWidget *koneconfig_key_combo_box_new(GtkLabel *label) {
	KoneconfigKeyComboBox *key_combo_box;
	KoneconfigKeyComboBoxClass *key_combo_box_class;
	KoneconfigKeyComboBoxPrivate *priv;

	key_combo_box = KONECONFIG_KEY_COMBO_BOX(g_object_new(KONECONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = KONECONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), key_combo_box_class->model);

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void koneconfig_key_combo_box_init(KoneconfigKeyComboBox *key_combo_box) {
	KoneconfigKeyComboBoxPrivate *priv = KONECONFIG_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Buttons", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 1", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 2", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 3", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 6", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_6, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 7", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_7, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 8", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_8, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 9", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_BUTTON_9, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Double click", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_DOUBLE_CLICK, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "OSD", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "OSD XY-Prescaling", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_OSD_XY_PRESCALING, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "OSD DPI", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_OSD_DPI, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "OSD Profile", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_OSD_PROFILE, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Key", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_KEY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_MACRO, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "DPI up", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_DPI_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "DPI down", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_DPI_DOWN, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Profile up", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Profile down", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_PROFILE_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Off", TYPE_COLUMN, KONE_BUTTON_INFO_TYPE_OFF, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	KoneconfigKeyComboBoxPrivate *priv = KONECONFIG_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(koneconfig_key_combo_box_parent_class)->finalize(object);
	g_free(priv->key_info);
}

static void koneconfig_key_combo_box_class_init(KoneconfigKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(KoneconfigKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void koneconfig_key_combo_box_set_value_blocked(KoneconfigKeyComboBox *key_combo_box, KoneRMPButtonInfo const *key_info) {
	KoneconfigKeyComboBoxPrivate *priv = key_combo_box->priv;

	g_free(priv->key_info);
	priv->key_info = kone_rmp_button_info_dup(key_info);
	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

KoneRMPButtonInfo *koneconfig_key_combo_box_get_value(KoneconfigKeyComboBox *key_combo_box) {
	return kone_rmp_button_info_dup(key_combo_box->priv->key_info);
}
