/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "koneplus_key_combo_box.h"
#include "roccat_talk_target_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_timer_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define KONEPLUS_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KONEPLUS_KEY_COMBO_BOX_TYPE, KoneplusKeyComboBoxClass))
#define IS_KONEPLUS_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KONEPLUS_KEY_COMBO_BOX_TYPE))
#define KONEPLUS_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), KONEPLUS_KEY_COMBO_BOX_TYPE, KoneplusKeyComboBoxClass))
#define KONEPLUS_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KONEPLUS_KEY_COMBO_BOX_TYPE, KoneplusKeyComboBoxPrivate))

typedef struct _KoneplusKeyComboBoxClass KoneplusKeyComboBoxClass;
typedef struct _KoneplusKeyComboBoxPrivate KoneplusKeyComboBoxPrivate;

struct _KoneplusKeyComboBox {
	GtkComboBox parent;
	KoneplusKeyComboBoxPrivate *priv;
};

struct _KoneplusKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _KoneplusKeyComboBoxPrivate {
	KoneplusRmpMacroKeyInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
	EASYSHIFT_ELEMENT_DISABLED = -3,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(KoneplusKeyComboBox, koneplus_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(KoneplusKeyComboBox *key_combo_box) {
	KoneplusKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(KoneplusRmpMacroKeyInfo const *macro_key_info) {
	return roccat_shortcut_dialog_create_name(roccat_get_bit(macro_key_info->keystrokes[0].action, KONEPLUS_PROFILE_BUTTON_MODIFIER_BIT_CTRL),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KONEPLUS_PROFILE_BUTTON_MODIFIER_BIT_SHIFT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KONEPLUS_PROFILE_BUTTON_MODIFIER_BIT_ALT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KONEPLUS_PROFILE_BUTTON_MODIFIER_BIT_WIN),
			macro_key_info->keystrokes[0].key);
}

static void set_label(KoneplusKeyComboBox *key_combo_box) {
	KoneplusKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext = NULL;

	switch (priv->key_info->type) {
	case KONEPLUS_PROFILE_BUTTON_TYPE_SHORTCUT:
		labeltext = shortcut_name(priv->key_info);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar const *)priv->key_info->macroset_name, (gchar const *)priv->key_info->macro_name);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH:
		labeltext = g_path_get_basename((gchar const *)priv->key_info->filename);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_TIMER:
		labeltext = g_strdup_printf("%s", (gchar const *)priv->key_info->timer_name);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT:
	case KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT_LOCK:
		labeltext = g_strdup(roccat_talk_target_dialog_get_label(koneplus_rmp_macro_key_info_get_talk_device(priv->key_info)));
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static KoneplusRmpMacroKeyInfo *button_quicklaunch(GtkWindow *parent) {
	GtkWidget *dialog;
	gchar *filename;
	KoneplusRmpMacroKeyInfo *key_info = NULL;

	dialog = gtk_file_chooser_dialog_new("Select executable",
			parent,
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL
	);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		key_info = koneplus_rmp_macro_key_info_new_quicklaunch(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
	return key_info;
}

static KoneplusRmpMacroKeyInfo *button_macro(GtkWindow *parent) {
	RoccatMacro *roccat_macro;
	KoneplusRmpMacroKeyInfo *key_info = NULL;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		key_info = roccat_macro_to_koneplus_rmp_macro_key_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static KoneplusRmpMacroKeyInfo *button_shortcut(GtkWindow *parent) {
	RoccatShortcutDialog *dialog;
	KoneplusRmpMacroKeyInfo *key_info = NULL;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = koneplus_rmp_macro_key_info_new_shortcut(
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				koneplus_rmp_macro_key_info_build_shortcut_modifier(
						roccat_shortcut_dialog_get_ctrl(dialog),
						roccat_shortcut_dialog_get_shift(dialog),
						roccat_shortcut_dialog_get_alt(dialog),
						roccat_shortcut_dialog_get_win(dialog)));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static KoneplusRmpMacroKeyInfo *button_timer(GtkWindow *parent) {
	KoneplusRmpMacroKeyInfo *key_info = NULL;
	RoccatTimer *timer;

	timer = roccat_timer_dialog(parent);

	if (timer) {
		key_info = koneplus_rmp_macro_key_info_new_timer(timer->name, roccat_timer_get_seconds(timer));
		roccat_timer_free(timer);
	}

	return key_info;
}

static KoneplusRmpMacroKeyInfo *button_talk_other(GtkWindow *parent, gint key_type) {
	guint target = roccat_talk_target_dialog(parent, ROCCAT_TALK_DEVICE_KONEPLUS);
	return koneplus_rmp_macro_key_info_new_talk(key_type, target);
}

static KoneplusRmpMacroKeyInfo *get_macro_key_info(GtkWindow *parent, gint type) {
	KoneplusRmpMacroKeyInfo *key_info;

	switch (type) {
	case KONEPLUS_PROFILE_BUTTON_TYPE_MACRO:
		key_info = button_macro(parent);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_SHORTCUT:
		key_info = button_shortcut(parent);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH:
		key_info = button_quicklaunch(parent);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_TIMER:
		key_info = button_timer(parent);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT:
	case KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT_LOCK:
		key_info = button_talk_other(parent, type);
		break;
	case KONEPLUS_PROFILE_BUTTON_TYPE_TALK_BOTH_EASYSHIFT:
		key_info = koneplus_rmp_macro_key_info_new_talk(type, ROCCAT_TALK_DEVICE_ALL);
		break;
	default:
		key_info = koneplus_rmp_macro_key_info_new_special(type);
	}

	return key_info;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	KoneplusKeyComboBox *key_combo_box = KONEPLUS_KEY_COMBO_BOX(combo);
	KoneplusKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	GtkTreeIter iter;
	gboolean valid;
	gint type_data;
	KoneplusRmpMacroKeyInfo *new_key_info;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type_data, -1);

	new_key_info = get_macro_key_info(parent, type_data);
	if (new_key_info) {
		koneplus_rmp_macro_key_info_free(priv->key_info);
		priv->key_info = new_key_info;
	} else
		set_active_iter(key_combo_box);

	set_label(key_combo_box);
}

static gboolean visible_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer pmask) {
	gint type;
	guint mask;

	mask = GPOINTER_TO_UINT(pmask);
	gtk_tree_model_get(model, iter, TYPE_COLUMN, &type, -1);

	if (mask & KONEPLUS_KEY_COMBO_BOX_NO_EASYSHIFT && (type == EASYSHIFT_ELEMENT_DISABLED || type == KONEPLUS_PROFILE_BUTTON_TYPE_EASYSHIFT))
		return FALSE;

	return TRUE;
}

static GtkTreeModel *filter(GtkTreeModel *child, guint mask) {
	GtkTreeModel *filter;

	filter = gtk_tree_model_filter_new(child, NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filter), visible_func, GUINT_TO_POINTER(mask), NULL);
	return filter;
}

GtkWidget *koneplus_key_combo_box_new(GtkLabel *label, guint mask) {
	KoneplusKeyComboBox *key_combo_box;
	KoneplusKeyComboBoxClass *key_combo_box_class;
	KoneplusKeyComboBoxPrivate *priv;

	key_combo_box = KONEPLUS_KEY_COMBO_BOX(g_object_new(KONEPLUS_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = KONEPLUS_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), filter(key_combo_box_class->model, mask));

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void koneplus_key_combo_box_init(KoneplusKeyComboBox *key_combo_box) {
	KoneplusKeyComboBoxPrivate *priv = KONEPLUS_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel, sublevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Buttons", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 1", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 2", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MENU, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 3", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_UNIVERSAL_SCROLLING, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 6", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TILT_RIGHT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 7", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TILT_LEFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 8", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_IE_BACKWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 9", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_IE_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Double click", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_DOUBLE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll up", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SCROLL_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll down", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SCROLL_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "CPI", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI cycle", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_CPI_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI up", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_CPI_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI down", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_CPI_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Profile", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile cycle", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_PROFILE_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile up", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile down", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_PROFILE_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity cycle", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity up", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity down", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easywheel", TYPE_COLUMN, EASYSHIFT_ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel Sensitivity", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_SENSITIVITY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel Profile", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_PROFILE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel CPI", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_CPI, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel Volume", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_VOLUME, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel Alt Tab", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_ALT_TAB, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Easywheel Aero Flip 3D", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_AERO_FLIP_3D, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easyshift", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, &sublevel, &toplevel, TITLE_COLUMN, "Self", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyshift", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYSHIFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyaim Setting 1", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYAIM_1, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyaim Setting 2", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYAIM_2, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyaim Setting 3", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYAIM_3, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyaim Setting 4", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYAIM_4, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Easyaim Setting 5", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_EASYAIM_5, -1);
	gtk_roccat_tree_store_append_with_values(store, &sublevel, &toplevel, TITLE_COLUMN, "Other", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Other Easyshift", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Other Easyshift lock", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TALK_OTHER_EASYSHIFT_LOCK, -1);
	gtk_roccat_tree_store_append_with_values(store, &sublevel, &toplevel, TITLE_COLUMN, "Both", TYPE_COLUMN, ELEMENT_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &sublevel, TITLE_COLUMN, "Both Easyshift", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TALK_BOTH_EASYSHIFT, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Timer", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Timer start", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TIMER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Timer stop", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_TIMER_STOP, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_MACRO, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Quicklaunch", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Windows key", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_WINDOWS_KEY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Open driver", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_OPEN_DRIVER, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Unused", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_UNUSED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, KONEPLUS_PROFILE_BUTTON_TYPE_DISABLED, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	KoneplusKeyComboBoxPrivate *priv = KONEPLUS_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(koneplus_key_combo_box_parent_class)->finalize(object);
	g_free(priv->key_info);
}

static void koneplus_key_combo_box_class_init(KoneplusKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(KoneplusKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void koneplus_key_combo_box_set_value_blocked(KoneplusKeyComboBox *key_combo_box, KoneplusRmpMacroKeyInfo const *key_info) {
	KoneplusKeyComboBoxPrivate *priv = key_combo_box->priv;

	koneplus_rmp_macro_key_info_free(priv->key_info);
	priv->key_info = koneplus_rmp_macro_key_info_dup(key_info);
	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

KoneplusRmpMacroKeyInfo *koneplus_key_combo_box_get_value(KoneplusKeyComboBox *key_combo_box) {
	return koneplus_rmp_macro_key_info_dup(key_combo_box->priv->key_info);
}
