/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "kovaplusconfig_key_combo_box.h"
#include "roccat/kovaplus.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_timer_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define KOVAPLUSCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), KOVAPLUSCONFIG_KEY_COMBO_BOX_TYPE, KovaplusconfigKeyComboBoxClass))
#define IS_KOVAPLUSCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), KOVAPLUSCONFIG_KEY_COMBO_BOX_TYPE))
#define KOVAPLUSCONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), KOVAPLUSCONFIG_KEY_COMBO_BOX_TYPE, KovaplusconfigKeyComboBoxClass))
#define KOVAPLUSCONFIG_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), KOVAPLUSCONFIG_KEY_COMBO_BOX_TYPE, KovaplusconfigKeyComboBoxPrivate))

typedef struct _KovaplusconfigKeyComboBoxClass KovaplusconfigKeyComboBoxClass;
typedef struct _KovaplusconfigKeyComboBoxPrivate KovaplusconfigKeyComboBoxPrivate;

struct _KovaplusconfigKeyComboBox {
	GtkComboBox parent;
	KovaplusconfigKeyComboBoxPrivate *priv;
};

struct _KovaplusconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _KovaplusconfigKeyComboBoxPrivate {
	KovaplusRmpMacroKeyInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
	EASYSHIFT_ELEMENT_DISABLED = -3,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(KovaplusconfigKeyComboBox, kovaplusconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(KovaplusconfigKeyComboBox *key_combo_box) {
	KovaplusconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(KovaplusRmpMacroKeyInfo const *macro_key_info) {
	return roccat_shortcut_dialog_create_name(roccat_get_bit(macro_key_info->keystrokes[0].action, KOVAPLUS_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_CTRL),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KOVAPLUS_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_SHIFT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KOVAPLUS_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_ALT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, KOVAPLUS_KEYSTROKE_SHORTCUT_ACTION_MODIFIER_BIT_WIN),
			macro_key_info->keystrokes[0].key);
}

static void set_label(KovaplusconfigKeyComboBox *key_combo_box) {
	KovaplusconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext = NULL;

	switch (priv->key_info->type) {
	case KOVAPLUS_PROFILE_BUTTON_TYPE_SHORTCUT:
		labeltext = shortcut_name(priv->key_info);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar const *)priv->key_info->macroset_name, (gchar const *)priv->key_info->macro_name);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH:
		labeltext = g_path_get_basename((gchar const *)priv->key_info->filename);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_TIMER:
		labeltext = g_strdup_printf("%s", (gchar const *)priv->key_info->timer_name);
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static KovaplusRmpMacroKeyInfo *button_quicklaunch(GtkWindow *parent) {
	GtkWidget *dialog;
	gchar *filename;
	KovaplusRmpMacroKeyInfo *key_info = NULL;

	dialog = gtk_file_chooser_dialog_new("Select executable",
			parent,
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL
	);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		key_info = kovaplus_rmp_macro_key_info_new_quicklaunch(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
	return key_info;
}

static KovaplusRmpMacroKeyInfo *button_macro(GtkWindow *parent) {
	RoccatMacro *roccat_macro;
	KovaplusRmpMacroKeyInfo *key_info = NULL;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		key_info = roccat_macro_to_kovaplus_rmp_macro_key_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static KovaplusRmpMacroKeyInfo *button_shortcut(GtkWindow *parent) {
	RoccatShortcutDialog *dialog;
	KovaplusRmpMacroKeyInfo *key_info = NULL;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = kovaplus_rmp_macro_key_info_new_shortcut(
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				kovaplus_rmp_macro_key_info_build_shortcut_modifier(
						roccat_shortcut_dialog_get_ctrl(dialog),
						roccat_shortcut_dialog_get_shift(dialog),
						roccat_shortcut_dialog_get_alt(dialog),
						roccat_shortcut_dialog_get_win(dialog)));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static KovaplusRmpMacroKeyInfo *button_timer(GtkWindow *parent) {
	KovaplusRmpMacroKeyInfo *key_info = NULL;
	RoccatTimer *timer;

	timer = roccat_timer_dialog(parent);

	if (timer) {
		key_info = kovaplus_rmp_macro_key_info_new_timer(timer->name, roccat_timer_get_seconds(timer));
		roccat_timer_free(timer);
	}

	return key_info;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	KovaplusconfigKeyComboBox *key_combo_box = KOVAPLUSCONFIG_KEY_COMBO_BOX(combo);
	KovaplusconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	GtkTreeIter iter;
	gboolean valid;
	gint type_data;
	KovaplusRmpMacroKeyInfo *new_key_info;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type_data, -1);

	switch (type_data) {
	case KOVAPLUS_PROFILE_BUTTON_TYPE_MACRO:
		new_key_info = button_macro(parent);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_SHORTCUT:
		new_key_info = button_shortcut(parent);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH:
		new_key_info = button_quicklaunch(parent);
		break;
	case KOVAPLUS_PROFILE_BUTTON_TYPE_TIMER:
		new_key_info = button_timer(parent);
		break;
	default:
		new_key_info = kovaplus_rmp_macro_key_info_new_special(type_data);
	}

	if (new_key_info) {
		g_free(priv->key_info);
		priv->key_info = new_key_info;
	} else
		set_active_iter(key_combo_box);

	set_label(key_combo_box);
}

static gboolean visible_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer pmask) {
	gint type;
	guint mask;

	mask = GPOINTER_TO_UINT(pmask);
	gtk_tree_model_get(model, iter, TYPE_COLUMN, &type, -1);
	if (mask & KOVAPLUSCONFIG_KEY_COMBO_BOX_NO_EASYSHIFT &&
			((type >= KOVAPLUS_PROFILE_BUTTON_TYPE_EASYSHIFT && type <= KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_ALT_TAB) ||
					type == EASYSHIFT_ELEMENT_DISABLED))
		return FALSE;

	return TRUE;
}

static GtkTreeModel *filter(GtkTreeModel *child, guint mask) {
	GtkTreeModel *filter;

	filter = gtk_tree_model_filter_new(child, NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filter), visible_func, GUINT_TO_POINTER(mask), NULL);
	return filter;
}

GtkWidget *kovaplusconfig_key_combo_box_new(GtkLabel *label, guint mask) {
	KovaplusconfigKeyComboBox *key_combo_box;
	KovaplusconfigKeyComboBoxClass *key_combo_box_class;
	KovaplusconfigKeyComboBoxPrivate *priv;

	key_combo_box = KOVAPLUSCONFIG_KEY_COMBO_BOX(g_object_new(KOVAPLUSCONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = KOVAPLUSCONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), filter(key_combo_box_class->model, mask));

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void kovaplusconfig_key_combo_box_init(KovaplusconfigKeyComboBox *key_combo_box) {
	KovaplusconfigKeyComboBoxPrivate *priv = KOVAPLUSCONFIG_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Buttons", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 1", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 2", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MENU, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 3", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_UNIVERSAL_SCROLLING, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 6", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_TILT_RIGHT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 7", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_TILT_LEFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 8", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_IE_BACKWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 9", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_IE_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Double click", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_DOUBLE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll up", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SCROLL_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll down", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SCROLL_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easywheel", TYPE_COLUMN, EASYSHIFT_ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_SENSITIVITY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_PROFILE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_CPI, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_VOLUME, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Alt Tab", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_ALT_TAB, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Aero Flip 3D", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYWHEEL_AERO_FLIP_3D, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Profile", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile cycle", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_PROFILE_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile up", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile down", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_PROFILE_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "CPI", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI cycle", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_CPI_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI up", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_CPI_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI down", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_CPI_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity cycle", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity up", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity down", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SENSITIVITY_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_MACRO, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Quicklaunch", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_QUICKLAUNCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Windows key", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_WINDOWS_KEY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Easyshift", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_EASYSHIFT, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Unused", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_UNUSED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Timer", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_TIMER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Timer stop", TYPE_COLUMN, KOVAPLUS_PROFILE_BUTTON_TYPE_TIMER_STOP, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	KovaplusconfigKeyComboBoxPrivate *priv = KOVAPLUSCONFIG_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(kovaplusconfig_key_combo_box_parent_class)->finalize(object);
	g_free(priv->key_info);
}

static void kovaplusconfig_key_combo_box_class_init(KovaplusconfigKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(KovaplusconfigKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void kovaplusconfig_key_combo_box_set_value_blocked(KovaplusconfigKeyComboBox *key_combo_box, KovaplusRmpMacroKeyInfo const *key_info) {
	KovaplusconfigKeyComboBoxPrivate *priv = key_combo_box->priv;

	g_free(priv->key_info);
	priv->key_info = kovaplus_rmp_macro_key_info_dup(key_info);
	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

KovaplusRmpMacroKeyInfo *kovaplusconfig_key_combo_box_get_value(KovaplusconfigKeyComboBox *key_combo_box) {
	return kovaplus_rmp_macro_key_info_dup(key_combo_box->priv->key_info);
}
