/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/roccat.h"
#include "g_roccat_helper.h"
#include <unistd.h>
#include <linux/input.h>

void roccat_input_event_write_single(int fd, int code) {
	roccat_input_event_write_with_sync(fd, code, 1);
	roccat_input_event_write_with_sync(fd, code, 0);
}

void roccat_input_event_write_with_sync(int fd, int code, int value) {
	struct input_event event;
	int written;

	if (fd == -1)
		return;

	event.type = EV_KEY;
	event.code = code;
	event.value = value;

	written = write(fd, (const void*) &event, sizeof(event));
	if (written != sizeof(event))
		g_warning("error with event %d", written);

	event.type = EV_SYN;
	event.code = SYN_REPORT;
	event.value = 0;

	written = write(fd, (const void*) &event, sizeof(event));
	if (written != sizeof(event))
		g_warning("error with sync event %d", written);
}

static guint8 roccat_keystroke_action_to_event_action(guint8 action) {
	return (action == ROCCAT_MACRO_KEYSTROKE_ACTION_PRESS) ? ROCCAT_INPUT_EVENT_VALUE_PRESS : ROCCAT_INPUT_EVENT_VALUE_RELEASE;
}

gboolean roccat_play_macro(int kbd_file, int mouse_file, RoccatMacro const *macro) {
	guint loop, key, count;
	RoccatMacroKeystroke const *keystroke;

	if (!macro)
		return FALSE;

	g_debug("Playing macro %s/%s", macro->macro_set, macro->macro);

	count = roccat_macro_keystrokes_get_count(&macro->keystrokes);

	for (loop = 0; loop < macro->keystrokes.loop; ++loop) {
		for (key = 0; key < count; ++key) {
			keystroke = &macro->keystrokes.keystrokes[key];
			if (roccat_hid_is_mouse_button(keystroke->key))
				roccat_input_event_write_with_sync(mouse_file, roccat_hid_to_btn_keycode(keystroke->key), roccat_keystroke_action_to_event_action(keystroke->action));
			else
				roccat_input_event_write_with_sync(kbd_file, roccat_hid_to_kbd_keycode(keystroke->key), roccat_keystroke_action_to_event_action(keystroke->action));
			usleep(roccat_macro_keystroke_get_period(keystroke) * G_ROCCAT_USEC_PER_MSEC);
		}
	}

	return TRUE;
}

gboolean roccat_play_macro_forked(pid_t *pid, int kbd_file, int mouse_file, RoccatMacro const *macro) {
	switch (roccat_fork_terminateable(pid)) {
	case 0:
		_exit(roccat_play_macro(kbd_file, mouse_file, macro) == TRUE ? 0 : 1);
		break;
	case -1:
		return FALSE;
	default:
		return TRUE;
	}
}
