/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/roccat.h"
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

struct _RoccatProcessMonitor {
	uid_t user_id;
	pid_t max_pid;
	gpointer user_data;
	guint timeout;
	gulong last_pid;
	RoccatProcessMonitorCreateCallback create_callback;
	RoccatProcessMonitorExitCallback exit_callback;
	GList *exit_subscriptions;
};

/* returns 0 on error */
static pid_t pid_get_actual(void) {
	GError *error = NULL;
	gchar *buffer;
	gchar *number_start;
	gboolean retval;
	gulong number_value;

	retval = g_file_get_contents("/proc/loadavg", &buffer, NULL, &error);
	if (!retval) {
		g_warning("g_file_get_contents(): %s", error->message);
		g_clear_error(&error);
		return 0;
	}

	number_start = strrchr(buffer, ' ');
	if (number_start == NULL) {
		g_warning("strrchr()");
		g_free(buffer);
		return 0;
	}

	/* playing it save here with data size, gets converted to target type on return */
	number_value = strtoul(number_start, NULL, 10);
	g_free(buffer);
	if ((number_value == 0) ||
			(number_value == ULONG_MAX && errno == ERANGE)) {
		g_warning("strtoul()");
		return 0;
	}

	return number_value;
}

static pid_t pid_get_max(void) {
	GError *error = NULL;
	gchar *buffer;
	gboolean retval;
	gulong number_value;

	retval = g_file_get_contents("/proc/sys/kernel/pid_max", &buffer, NULL, &error);
	if (!retval) {
		g_warning("get_pid_max(): g_file_get_contents(): %s", error->message);
		g_clear_error(&error);
		return 0;
	}

	/* playing it save here with data size, gets converted to target type on return */
	number_value = strtoul(buffer, NULL, 10);
	g_free(buffer);
	if ((number_value == 0) ||
			(number_value == ULONG_MAX && errno == ERANGE)) {
		g_warning("get_pid_max(): strtoul()");
		return 0;
	}

	return number_value;
}

/*
 * This is not to be used with root rights.
 * returns 0 on error or if uid is indeed from root
 */
static uid_t pid_get_uid(pid_t pid) {
	gchar *dirname;
	struct stat status;
	gint retval;

	dirname = g_strdup_printf("/proc/%li", (gulong)pid);
	retval = stat(dirname, &status);
	g_free(dirname);

	if (retval == -1)
		return 0;

	return status.st_uid;
}

// FIXME put in g_roccat_helper
static gchar *g_roccat_file_read_link_utf8(gchar const *filename, GError **error) {
	gchar *link;
	gchar *utf8_link;

	link = g_file_read_link(filename, error);
	if (*error)
		return NULL;

	utf8_link = g_filename_to_utf8(link, -1, NULL, NULL, error);
	g_free(link);
	if (*error)
		return NULL;

	return utf8_link;
}

static gchar *pid_get_link(pid_t pid, GError **error) {
	gchar *filename;
	gchar *link;

	filename = g_strdup_printf("/proc/%li/exe", (gulong)pid);
	link = g_roccat_file_read_link_utf8(filename, error);
	g_free(filename);
	return link;
}

static gboolean pid_get_cmdline(pid_t pid, gchar **buffer, gsize *length, GError **error) {
	gchar *filename;
	gboolean retval;

	filename = g_strdup_printf("/proc/%li/cmdline", (gulong)pid);
	retval = g_file_get_contents(filename, buffer, length, error);
	g_free(filename);
	return retval;
}

/* read cmdline and expand executable with absolute path from exe-link */
static gchar *pid_get_abs_cmdline_if_user(pid_t pid, uid_t uid, GError **error) {
	gchar *cmdline;
	gchar *link;
	gsize length;
	gchar *abs;
	gchar *parameter_start = NULL;
	gsize iterator;

	if (pid_get_uid(pid) != uid) {
		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_ACCES, "wrong uid");
		return NULL;
	}

	link = pid_get_link(pid, error);
	if (*error)
		return NULL;

	pid_get_cmdline(pid, &cmdline, &length, error);
	if (*error) {
		g_free(link);
		return NULL;
	}

	if (length == 0) {
		g_debug("(%s:%li): cmdline has 0 length", link, (gulong)pid);
		abs = g_strdup(link);
		g_free(cmdline);
		g_free(link);
		return abs;
	}

	/* make length string length, not buffer length */
	--length;

	/* set parameter_start to first char after first 0 in cmdline */
	for (iterator = 0; iterator < length; ++iterator) {
		if (cmdline[iterator] == 0) {
			parameter_start = cmdline + iterator + 1;
			break;
		}
	}

	/* happens when no args */
	if (parameter_start == NULL)
		abs = g_strdup(link);
	else {
		/* substitute 0 delimiter with spaces except eos */
		for (iterator = 0; iterator < length; ++iterator) {
			if (cmdline[iterator] == 0)
				cmdline[iterator] = ' ';
		}

		abs = g_strdup_printf("%s %s", link, parameter_start);
	}

	g_free(cmdline);
	g_free(link);

	return abs;
}

static void try_pid(RoccatProcessMonitor *monitor, pid_t pid) {
	gchar *string;
	GError *error = NULL;

	string = pid_get_abs_cmdline_if_user(pid, monitor->user_id, &error);
	if (error) {
		if (!((g_error_matches(error, g_file_error_quark(), G_FILE_ERROR_NOENT)) || /* we where to slow */
				(g_error_matches(error, g_file_error_quark(), G_FILE_ERROR_ACCES)) /* it's definitely not from us */
				)) {
			g_warning("%s", error->message);
		}
		g_clear_error(&error);
		return;
	}

	if (monitor->create_callback(pid, string, monitor->user_data))
		monitor->exit_subscriptions = g_list_append(monitor->exit_subscriptions, (gpointer)(gulong)pid);

	g_free(string);
}

static void scan_proc(RoccatProcessMonitor *monitor, pid_t start_pid, pid_t stop_pid) {
	pid_t actual_pid;

	if (stop_pid > start_pid) {
		for (actual_pid = start_pid + 1; actual_pid <= stop_pid; ++actual_pid)
			try_pid(monitor, actual_pid);
	} else if (stop_pid < start_pid) { /* overflow in id's */
		for (actual_pid = start_pid + 1; actual_pid <= monitor->max_pid; ++actual_pid)
			try_pid(monitor, actual_pid);
		for (actual_pid = 1; actual_pid <= stop_pid; ++actual_pid)
			try_pid(monitor, actual_pid);
	}
}

static void scan_exit_subscriptions(RoccatProcessMonitor *monitor) {
	GList *element;
	GList *to_remove = NULL;
	pid_t pid;
	gchar *filename;

	for (element = g_list_first(monitor->exit_subscriptions); element; element = g_list_next(element)) {
		pid = (pid_t)(gulong)element->data;
		filename = g_strdup_printf("/proc/%li", (gulong)pid);

		if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
			monitor->exit_callback(pid, monitor->user_data);
			to_remove = g_list_append(to_remove, element);
		}

		g_free(filename);
	}

	for (element = g_list_first(to_remove); element; element = g_list_next(element))
		monitor->exit_subscriptions = g_list_delete_link(monitor->exit_subscriptions, element->data);

	g_list_free(to_remove);
}

static gboolean timeout_cb(RoccatProcessMonitor *monitor) {
	pid_t pid;

	pid = pid_get_actual();

	/* nothing to do */
	if (pid == monitor->last_pid)
		return TRUE;

	/* don't scan already started processes */
	if (monitor->last_pid != 0)
		scan_proc(monitor, monitor->last_pid, pid);

	scan_exit_subscriptions(monitor);

	monitor->last_pid = pid;

	return TRUE;
}

RoccatProcessMonitor *roccat_process_monitor_new(guint interval,
		RoccatProcessMonitorCreateCallback create_callback,
				RoccatProcessMonitorExitCallback exit_callback, gpointer user_data) {
	RoccatProcessMonitor *monitor;

	monitor = (RoccatProcessMonitor *)g_malloc0(sizeof(RoccatProcessMonitor));

	/* TODO max pid can change */
	monitor->max_pid = pid_get_max();
	monitor->user_id = getuid();
	monitor->user_data = user_data;
	monitor->create_callback = create_callback;
	monitor->exit_callback = exit_callback;
	monitor->timeout = g_timeout_add(interval, (GSourceFunc)timeout_cb, monitor);

	return monitor;
}

void roccat_process_monitor_free(RoccatProcessMonitor *monitor) {
	if (!monitor)
		return;
	g_source_remove(monitor->timeout);
	g_list_free(monitor->exit_subscriptions);
	g_free(monitor);
}
