/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "roccat/roccat.h"
#include "g_roccat_helper.h"
#include <string.h>

typedef struct {
	pid_t pid;
	gchar *path;
	guint profile_index;
} ListElement;

static ListElement *list_element_new(pid_t pid, gchar const *path, guint profile_index) {
	ListElement *element;

	element = (ListElement *)g_malloc(sizeof(ListElement));
	element->pid = pid;
	element->path = g_strdup(path);
	element->profile_index = profile_index;

	return element;
}

static ListElement *list_element_manual_new(guint profile_index) {
	return list_element_new(0, "", profile_index);
}

static void list_element_free(ListElement *element) {
	g_free(element->path);
	g_free(element);
}

static gint list_element_find_by_pid(gconstpointer a, gconstpointer b) {
	return ((ListElement *)a)->pid - (pid_t)(gulong)b;
}

static gint list_element_find_by_path(gconstpointer a, gconstpointer b) {
	return strcmp(((ListElement *)a)->path, (gchar const *)b);
}

struct _RoccatProfileCascade {
	GList *list;
};

RoccatProfileCascade *roccat_profile_cascade_new(guint actual_profile_index) {
	RoccatProfileCascade *cascade;

	cascade = (RoccatProfileCascade *)g_malloc0(sizeof(RoccatProfileCascade));
	cascade->list = g_list_append(cascade->list, list_element_manual_new(actual_profile_index));

	return cascade;
}

static void roccat_profile_cascade_free_list(RoccatProfileCascade *cascade) {
	g_roccat_list_free_full(cascade->list, (GDestroyNotify)list_element_free);
	cascade->list = NULL;
}

void roccat_profile_cascade_free(RoccatProfileCascade *cascade) {
	if (!cascade)
		return;
	roccat_profile_cascade_free_list(cascade);
	g_free(cascade);
}

static void roccat_profile_cascade_remove_element(RoccatProfileCascade *cascade, GList *element) {
	cascade->list = g_list_remove_link(cascade->list, element);
	g_roccat_list_free_full(element, (GDestroyNotify)list_element_free);
}

gint roccat_profile_cascade_on_profile_change(RoccatProfileCascade *cascade, guint new_profile_index) {
	GList *last;
	guint last_profile_index;

	last = g_list_last(cascade->list);
	if (last == NULL)
		goto create_new;

	last_profile_index = ((ListElement *)last->data)->profile_index;

	if (last_profile_index != new_profile_index)
		goto empty_list;
	else
		goto do_nothing;

empty_list:
	roccat_profile_cascade_free_list(cascade);
create_new:
	cascade->list = g_list_append(cascade->list, list_element_manual_new(new_profile_index));
do_nothing:
	return new_profile_index;
}

gint roccat_profile_cascade_application_start(RoccatProfileCascade *cascade, pid_t pid, gchar const *path, guint profile_index) {
	/* don't do anything if path is already present */
	if (g_list_find_custom(cascade->list, path, list_element_find_by_path))
		return -1;

	cascade->list = g_list_append(cascade->list, list_element_new(pid, path, profile_index));
	return profile_index;
}

gint roccat_profile_cascade_application_stop(RoccatProfileCascade *cascade, pid_t pid) {
	GList *element;
	GList *previous;
	guint retval;

	if (pid == 0)
		return -1;

	element = g_list_find_custom(cascade->list, (gpointer)(gulong)pid, list_element_find_by_pid);
	if (element == NULL)
		return -1;

	if (element == g_list_last(cascade->list))  {
		previous = g_list_previous(element);

		if (previous == NULL)
			retval = -1;
		else
			retval = ((ListElement *)(previous->data))->profile_index;
	} else
		retval = -1;

	roccat_profile_cascade_remove_element(cascade, element);
	return retval;
}
