/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "pyraconfig_key_combo_box.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define PYRACONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), PYRACONFIG_KEY_COMBO_BOX_TYPE, PyraconfigKeyComboBoxClass))
#define IS_PYRACONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), PYRACONFIG_KEY_COMBO_BOX_TYPE))
#define PYRACONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), PYRACONFIG_KEY_COMBO_BOX_TYPE, PyraconfigKeyComboBoxClass))
#define PYRACONFIG_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), PYRACONFIG_KEY_COMBO_BOX_TYPE, PyraconfigKeyComboBoxPrivate))

typedef struct _PyraconfigKeyComboBoxClass PyraconfigKeyComboBoxClass;
typedef struct _PyraconfigKeyComboBoxPrivate PyraconfigKeyComboBoxPrivate;

struct _PyraconfigKeyComboBox {
	GtkComboBox parent;
	PyraconfigKeyComboBoxPrivate *priv;
};

struct _PyraconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _PyraconfigKeyComboBoxPrivate {
	PyraRMPMacroKeyInfo *key_info;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(PyraconfigKeyComboBox, pyraconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(PyraconfigKeyComboBox *key_combo_box) {
	PyraconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->key_info->type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(PyraRMPMacroKeyInfo const *macro_key_info) {
	return roccat_shortcut_dialog_create_name(roccat_get_bit(macro_key_info->keystrokes[0].action, PYRA_RMP_KEYSTROKE_SHORTCUT_ACTION_CTRL),
			roccat_get_bit(macro_key_info->keystrokes[0].action, PYRA_RMP_KEYSTROKE_SHORTCUT_ACTION_SHIFT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, PYRA_RMP_KEYSTROKE_SHORTCUT_ACTION_ALT),
			roccat_get_bit(macro_key_info->keystrokes[0].action, PYRA_RMP_KEYSTROKE_SHORTCUT_ACTION_WIN),
			macro_key_info->keystrokes[0].key);
}

static void set_label(PyraconfigKeyComboBox *key_combo_box) {
	PyraconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext = NULL;

	switch (priv->key_info->type) {
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_SHORTCUT:
		labeltext = shortcut_name(priv->key_info);
		break;
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_MACRO:
		labeltext = g_strdup_printf("%s/%s", (gchar const *)priv->key_info->macro_set_name, (gchar const *)priv->key_info->macro_name);
		break;
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_QUICKLAUNCH:
		labeltext = g_path_get_basename((gchar const *)priv->key_info->filename);
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static PyraRMPMacroKeyInfo *button_quicklaunch(GtkWindow *parent) {
	GtkWidget *dialog;
	gchar *filename;
	PyraRMPMacroKeyInfo *key_info = NULL;

	dialog = gtk_file_chooser_dialog_new("Select executable", parent,
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		key_info = pyra_rmp_macro_key_info_new_quicklaunch(filename);
		g_free(filename);
	}
	gtk_widget_destroy(dialog);
	return key_info;
}

static PyraRMPMacroKeyInfo *button_macro(GtkWindow *parent) {
	RoccatMacro *roccat_macro;
	PyraRMPMacroKeyInfo *key_info = NULL;

	roccat_macro = roccat_macro_dialog(parent);
	if (roccat_macro) {
		key_info = roccat_macro_to_pyra_rmp_macro_key_info(roccat_macro);
		g_free(roccat_macro);
	}
	return key_info;
}

static PyraRMPMacroKeyInfo *button_shortcut(GtkWindow *parent) {
	RoccatShortcutDialog *dialog;
	PyraRMPMacroKeyInfo *key_info = NULL;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(parent));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		key_info = pyra_rmp_macro_key_info_new_shortcut(
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				roccat_shortcut_dialog_get_ctrl(dialog),
				roccat_shortcut_dialog_get_shift(dialog),
				roccat_shortcut_dialog_get_alt(dialog),
				roccat_shortcut_dialog_get_win(dialog));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
	return key_info;
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	PyraconfigKeyComboBox *key_combo_box = PYRACONFIG_KEY_COMBO_BOX(combo);
	PyraconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWindow *parent = gtk_roccat_widget_get_gtk_window(GTK_WIDGET(combo));
	GtkTreeIter iter;
	gboolean valid;
	guint type_data;
	PyraRMPMacroKeyInfo *new_key_info;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type_data, -1);

	switch (type_data) {
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_MACRO:
		new_key_info = button_macro(parent);
		break;
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_SHORTCUT:
		new_key_info = button_shortcut(parent);
		break;
	case PYRA_RMP_MACRO_KEY_INFO_TYPE_QUICKLAUNCH:
		new_key_info = button_quicklaunch(parent);
		break;
	default:
		new_key_info = pyra_rmp_macro_key_info_new_special(type_data);
	}

	if (new_key_info) {
		g_free(priv->key_info);
		priv->key_info = new_key_info;
	} else
		set_active_iter(key_combo_box);

	set_label(key_combo_box);
}

GtkWidget *pyraconfig_key_combo_box_new(GtkLabel *label) {
	PyraconfigKeyComboBox *key_combo_box;
	PyraconfigKeyComboBoxClass *key_combo_box_class;
	PyraconfigKeyComboBoxPrivate *priv;

	key_combo_box = PYRACONFIG_KEY_COMBO_BOX(g_object_new(PYRACONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = PYRACONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), key_combo_box_class->model);

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void pyraconfig_key_combo_box_init(PyraconfigKeyComboBox *key_combo_box) {
	PyraconfigKeyComboBoxPrivate *priv = PYRACONFIG_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Buttons", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 1", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 2", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MENU, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 3", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_UNIVERSAL_SCROLLING, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 6", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_TILT_RIGHT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 7", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_TILT_LEFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 8", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_IE_BACKWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 9", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_IE_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Double click", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_DOUBLE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll up", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_SCROLL_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll down", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_SCROLL_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easywheel", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_EASYWHEEL_SENSITIVITY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_EASYWHEEL_PROFILE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_EASYWHEEL_CPI, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_EASYWHEEL_VOLUME, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Profile", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile cycle", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_PROFILE_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile up", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile down", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_PROFILE_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "CPI", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI cycle", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_CPI_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI up", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_CPI_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI down", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_CPI_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_MACRO, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Quicklaunch", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_QUICKLAUNCH, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Easyshift", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_EASYSHIFT, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Unused", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_UNUSED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_DISABLED, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, PYRA_RMP_MACRO_KEY_INFO_TYPE_DISABLED2, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	PyraconfigKeyComboBoxPrivate *priv = PYRACONFIG_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(pyraconfig_key_combo_box_parent_class)->finalize(object);
	g_free(priv->key_info);
}

static void pyraconfig_key_combo_box_class_init(PyraconfigKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(PyraconfigKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void pyraconfig_key_combo_box_set_value_blocked(PyraconfigKeyComboBox *key_combo_box, PyraRMPMacroKeyInfo const *key_info) {
	PyraconfigKeyComboBoxPrivate *priv = key_combo_box->priv;

	g_free(priv->key_info);
	priv->key_info = pyra_rmp_macro_key_info_dup(key_info);
	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

PyraRMPMacroKeyInfo *pyraconfig_key_combo_box_get_value(PyraconfigKeyComboBox *key_combo_box) {
	return pyra_rmp_macro_key_info_dup(key_combo_box->priv->key_info);
}
