/*
 * This file is part of roccat-tools.
 *
 * roccat-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * roccat-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with roccat-tools. If not, see <http://www.gnu.org/licenses/>.
 */

#include "savuconfig_key_combo_box.h"
#include "roccat_shortcut_dialog.h"
#include "roccat_timer_dialog.h"
#include "roccat_macro_dialog.h"
#include "roccat_gdk_key_translations.h"
#include "gtk_roccat_helper.h"

#define SAVUCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST((klass), SAVUCONFIG_KEY_COMBO_BOX_TYPE, SavuconfigKeyComboBoxClass))
#define IS_SAVUCONFIG_KEY_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), SAVUCONFIG_KEY_COMBO_BOX_TYPE))
#define SAVUCONFIG_KEY_COMBO_BOX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), SAVUCONFIG_KEY_COMBO_BOX_TYPE, SavuconfigKeyComboBoxClass))
#define SAVUCONFIG_KEY_COMBO_BOX_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), SAVUCONFIG_KEY_COMBO_BOX_TYPE, SavuconfigKeyComboBoxPrivate))

typedef struct _SavuconfigKeyComboBoxClass SavuconfigKeyComboBoxClass;
typedef struct _SavuconfigKeyComboBoxPrivate SavuconfigKeyComboBoxPrivate;

struct _SavuconfigKeyComboBox {
	GtkComboBox parent;
	SavuconfigKeyComboBoxPrivate *priv;
};

struct _SavuconfigKeyComboBoxClass {
	GtkComboBoxClass parent_class;
	GtkTreeModel *model;
};

struct _SavuconfigKeyComboBoxPrivate {
	SavuButton button;
	RoccatMacro *macro;
	RoccatTimer *timer;
	gchar *launchpath;
	GtkLabel *label;
	gulong changed_handler_id;
};

enum {
	ELEMENT_DISABLED = -2,
	EASYSHIFT_ELEMENT_DISABLED = -3,
};

enum {
	TITLE_COLUMN,
	TYPE_COLUMN,
	N_COLUMNS
};

G_DEFINE_TYPE(SavuconfigKeyComboBox, savuconfig_key_combo_box, GTK_TYPE_COMBO_BOX);

static void set_active_iter(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkTreeIter iter;
	gboolean valid;

	valid = gtk_roccat_tree_model_iter_find_int(gtk_combo_box_get_model(GTK_COMBO_BOX(key_combo_box)),
			TYPE_COLUMN, priv->button.type, &iter, NULL);
	if (valid) {
		g_signal_handler_block(G_OBJECT(key_combo_box), priv->changed_handler_id);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(key_combo_box), &iter);
		g_signal_handler_unblock(G_OBJECT(key_combo_box), priv->changed_handler_id);
	}
}

static gchar *shortcut_name(SavuButton const *button) {
	return roccat_shortcut_dialog_create_name(roccat_get_bit(button->modifier, SAVU_BUTTON_MODIFIER_BIT_CTRL),
			roccat_get_bit(button->modifier, SAVU_BUTTON_MODIFIER_BIT_SHIFT),
			roccat_get_bit(button->modifier, SAVU_BUTTON_MODIFIER_BIT_ALT),
			roccat_get_bit(button->modifier, SAVU_BUTTON_MODIFIER_BIT_WIN),
			button->key);
}

static void set_label(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	gchar *labeltext = NULL;

	switch (priv->button.type) {
	case SAVU_BUTTON_TYPE_SHORTCUT:
		labeltext = shortcut_name(&priv->button);
		break;
	case SAVU_BUTTON_TYPE_MACRO:
		labeltext = roccat_macro_get_name_joined(priv->macro);
		break;
	case SAVU_BUTTON_TYPE_QUICKLAUNCH:
		labeltext = g_path_get_basename(priv->launchpath);
		break;
	case SAVU_BUTTON_TYPE_TIMER:
		labeltext = g_strdup(priv->timer->name);
		break;
	default:
		labeltext = NULL;
	}
	gtk_label_set_text(priv->label, labeltext);
	g_free(labeltext);
}

static void button_quicklaunch(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	GtkWidget *dialog;

	dialog = gtk_file_chooser_dialog_new("Select executable",
			gtk_roccat_widget_get_gtk_window(GTK_WIDGET(key_combo_box)),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			NULL);

	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		savu_button_set_to_normal(&priv->button, SAVU_BUTTON_TYPE_QUICKLAUNCH);
		g_free(priv->launchpath);
		priv->launchpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
	}
	gtk_widget_destroy(dialog);
}

static void button_macro(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	RoccatMacro *roccat_macro;

	roccat_macro = roccat_macro_dialog(gtk_roccat_widget_get_gtk_window(GTK_WIDGET(key_combo_box)));
	if (roccat_macro) {
		savu_button_set_to_normal(&priv->button, SAVU_BUTTON_TYPE_MACRO);
		roccat_macro_free(priv->macro);
		priv->macro = roccat_macro;
	}
}

static void button_shortcut(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	RoccatShortcutDialog *dialog;

	dialog = ROCCAT_SHORTCUT_DIALOG(roccat_shortcut_dialog_new(gtk_roccat_widget_get_gtk_window(GTK_WIDGET(key_combo_box))));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
		savu_button_set_to_shortcut(&priv->button,
				roccat_keycode_to_hid(roccat_shortcut_dialog_get_keycode(dialog)),
				savu_button_build_shortcut_modifier(
						roccat_shortcut_dialog_get_ctrl(dialog),
						roccat_shortcut_dialog_get_shift(dialog),
						roccat_shortcut_dialog_get_alt(dialog),
						roccat_shortcut_dialog_get_win(dialog)));
	}
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

static void button_timer(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	RoccatTimer *roccat_timer;

	roccat_timer = roccat_timer_dialog(gtk_roccat_widget_get_gtk_window(GTK_WIDGET(key_combo_box)));
	if (roccat_timer) {
		savu_button_set_to_normal(&priv->button, SAVU_BUTTON_TYPE_TIMER);
		roccat_timer_free(priv->timer);
		priv->timer = roccat_timer;
	}
}

static void button_normal(SavuconfigKeyComboBox *key_combo_box, gint type) {
	savu_button_set_to_normal(&key_combo_box->priv->button, type);
}

static void button_changed_cb(GtkComboBox *combo, gpointer user_data) {
	SavuconfigKeyComboBox *key_combo_box = SAVUCONFIG_KEY_COMBO_BOX(combo);
	GtkTreeIter iter;
	gboolean valid;
	gint type;

	valid = gtk_combo_box_get_active_iter(combo, &iter);
	if (!valid)
		return;

	gtk_tree_model_get(gtk_combo_box_get_model(combo), &iter, TYPE_COLUMN, &type, -1);

	switch (type) {
	case SAVU_BUTTON_TYPE_MACRO:
		button_macro(key_combo_box);
		break;
	case SAVU_BUTTON_TYPE_SHORTCUT:
		button_shortcut(key_combo_box);
		break;
	case SAVU_BUTTON_TYPE_QUICKLAUNCH:
		button_quicklaunch(key_combo_box);
		break;
	case SAVU_BUTTON_TYPE_TIMER:
		button_timer(key_combo_box);
		break;
	default:
		button_normal(key_combo_box, type);
	}

	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

static gboolean visible_func(GtkTreeModel *model, GtkTreeIter *iter, gpointer pmask) {
	gint type;
	guint mask;

	mask = GPOINTER_TO_UINT(pmask);
	gtk_tree_model_get(model, iter, TYPE_COLUMN, &type, -1);

	if (mask & SAVUCONFIG_KEY_COMBO_BOX_NO_EASYSHIFT && (type == EASYSHIFT_ELEMENT_DISABLED || type == SAVU_BUTTON_TYPE_EASYSHIFT))
		return FALSE;

	return TRUE;
}

static GtkTreeModel *filter(GtkTreeModel *child, guint mask) {
	GtkTreeModel *filter;

	filter = gtk_tree_model_filter_new(child, NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filter), visible_func, GUINT_TO_POINTER(mask), NULL);
	return filter;
}

GtkWidget *savuconfig_key_combo_box_new(GtkLabel *label, guint mask) {
	SavuconfigKeyComboBox *key_combo_box;
	SavuconfigKeyComboBoxClass *key_combo_box_class;
	SavuconfigKeyComboBoxPrivate *priv;

	key_combo_box = SAVUCONFIG_KEY_COMBO_BOX(g_object_new(SAVUCONFIG_KEY_COMBO_BOX_TYPE,
			NULL));

	priv = key_combo_box->priv;

	priv->label = label;

	key_combo_box_class = SAVUCONFIG_KEY_COMBO_BOX_GET_CLASS(key_combo_box);
	gtk_combo_box_set_model(GTK_COMBO_BOX(key_combo_box), filter(key_combo_box_class->model, mask));

	priv->changed_handler_id = g_signal_connect(G_OBJECT(key_combo_box), "changed", G_CALLBACK(button_changed_cb), NULL);

	return GTK_WIDGET(key_combo_box);
}

static void cell_data_func(GtkCellLayout *cell_layout, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, gpointer data) {
	gboolean sensitive;

	/* disable toplevel elements */
	sensitive = !gtk_tree_model_iter_has_child(model, iter);
	g_object_set(cell, "sensitive", sensitive, NULL);
}

static void savuconfig_key_combo_box_init(SavuconfigKeyComboBox *key_combo_box) {
	SavuconfigKeyComboBoxPrivate *priv = SAVUCONFIG_KEY_COMBO_BOX_GET_PRIVATE(key_combo_box);
	GtkCellRenderer *cell;

	key_combo_box->priv = priv;

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(key_combo_box), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(key_combo_box), cell, "text", TITLE_COLUMN, NULL);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(key_combo_box), cell, cell_data_func, NULL, NULL);
}

static GtkTreeModel *button_info_model_new(void) {
	GtkTreeStore *store;
	GtkTreeIter toplevel;

	store = gtk_tree_store_new(N_COLUMNS, G_TYPE_STRING, G_TYPE_INT);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Buttons", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 1", TYPE_COLUMN, SAVU_BUTTON_TYPE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 2", TYPE_COLUMN, SAVU_BUTTON_TYPE_MENU, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 3", TYPE_COLUMN, SAVU_BUTTON_TYPE_UNIVERSAL_SCROLLING, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 6", TYPE_COLUMN, SAVU_BUTTON_TYPE_TILT_RIGHT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 7", TYPE_COLUMN, SAVU_BUTTON_TYPE_TILT_LEFT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 8", TYPE_COLUMN, SAVU_BUTTON_TYPE_IE_BACKWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Button 9", TYPE_COLUMN, SAVU_BUTTON_TYPE_IE_FORWARD, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Double click", TYPE_COLUMN, SAVU_BUTTON_TYPE_DOUBLE_CLICK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll up", TYPE_COLUMN, SAVU_BUTTON_TYPE_SCROLL_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Scroll down", TYPE_COLUMN, SAVU_BUTTON_TYPE_SCROLL_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "CPI", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI cycle", TYPE_COLUMN, SAVU_BUTTON_TYPE_CPI_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI up", TYPE_COLUMN, SAVU_BUTTON_TYPE_CPI_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI down", TYPE_COLUMN, SAVU_BUTTON_TYPE_CPI_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easyaim", TYPE_COLUMN, EASYSHIFT_ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "400 cpi", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYAIM_400, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "800 cpi", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYAIM_800, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "1600 cpi", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYAIM_1600, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "4000 cpi", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYAIM_4000, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Easywheel", TYPE_COLUMN, EASYSHIFT_ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_SENSITIVITY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_PROFILE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "CPI", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_CPI, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_VOLUME, -1);
//	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Alt Tab", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_ALT_TAB, -1);
//	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Aero Flip 3D", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYWHEEL_AERO_FLIP_3D, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Multimedia", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Open player", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_OPEN_PLAYER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Previous track", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_PREV_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Next track", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_NEXT_TRACK, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Play/Pause", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_PLAY_PAUSE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Stop", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_STOP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Mute", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_MUTE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume up", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_VOLUME_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Volume down", TYPE_COLUMN, SAVU_BUTTON_TYPE_MULTIMEDIA_VOLUME_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Profile", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile cycle", TYPE_COLUMN, SAVU_BUTTON_TYPE_PROFILE_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile up", TYPE_COLUMN, SAVU_BUTTON_TYPE_PROFILE_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Profile down", TYPE_COLUMN, SAVU_BUTTON_TYPE_PROFILE_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Sensitivity", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity cycle", TYPE_COLUMN, SAVU_BUTTON_TYPE_SENSITIVITY_CYCLE, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity up", TYPE_COLUMN, SAVU_BUTTON_TYPE_SENSITIVITY_UP, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Sensitivity down", TYPE_COLUMN, SAVU_BUTTON_TYPE_SENSITIVITY_DOWN, -1);

	gtk_roccat_tree_store_append_with_values(store, &toplevel, NULL, TITLE_COLUMN, "Timer", TYPE_COLUMN, ELEMENT_DISABLED, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Timer start", TYPE_COLUMN, SAVU_BUTTON_TYPE_TIMER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, &toplevel, TITLE_COLUMN, "Timer stop", TYPE_COLUMN, SAVU_BUTTON_TYPE_TIMER_STOP, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Shortcut", TYPE_COLUMN, SAVU_BUTTON_TYPE_SHORTCUT, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Macro", TYPE_COLUMN, SAVU_BUTTON_TYPE_MACRO, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Quicklaunch", TYPE_COLUMN, SAVU_BUTTON_TYPE_QUICKLAUNCH, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Windows key", TYPE_COLUMN, SAVU_BUTTON_TYPE_WINDOWS_KEY, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Open driver", TYPE_COLUMN, SAVU_BUTTON_TYPE_OPEN_DRIVER, -1);
	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Easyshift", TYPE_COLUMN, SAVU_BUTTON_TYPE_EASYSHIFT, -1);

	gtk_roccat_tree_store_append_with_values(store, NULL, NULL, TITLE_COLUMN, "Disabled", TYPE_COLUMN, SAVU_BUTTON_TYPE_DISABLED, -1);

	return GTK_TREE_MODEL(store);
}

static void finalize(GObject *object) {
	SavuconfigKeyComboBoxPrivate *priv = SAVUCONFIG_KEY_COMBO_BOX(object)->priv;
	G_OBJECT_CLASS(savuconfig_key_combo_box_parent_class)->finalize(object);
	roccat_macro_free(priv->macro);
	roccat_timer_free(priv->timer);
	g_free(priv->launchpath);
}

static void savuconfig_key_combo_box_class_init(SavuconfigKeyComboBoxClass *klass) {
	GObjectClass *gobject_class = (GObjectClass*)klass;

	gobject_class->finalize = finalize;
	g_type_class_add_private(klass, sizeof(SavuconfigKeyComboBoxPrivate));
	klass->model = button_info_model_new();
}

void savuconfig_key_combo_box_set_from_rmp(SavuconfigKeyComboBox *key_combo_box, SavuRmp *rmp, guint index) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	SavuButton *button;
	SavuRmpMacroKeyInfo *macro;
	SavuRmpTimer *timer;

	button = savu_rmp_get_button(rmp, index);
	priv->button = *button;
	g_free(button);

	switch (priv->button.type) {
	case SAVU_BUTTON_TYPE_MACRO:
		macro = savu_rmp_get_macro_key_info(rmp, index);
		roccat_macro_free(priv->macro);
		priv->macro = savu_rmp_macro_key_info_to_roccat_macro(macro);
		savu_rmp_macro_key_info_free(macro);
		break;
	case SAVU_BUTTON_TYPE_QUICKLAUNCH:
		g_free(priv->launchpath);
		priv->launchpath = savu_rmp_get_launchpath(rmp, index);
		break;
	case SAVU_BUTTON_TYPE_TIMER:
		timer = savu_rmp_get_timer(rmp, index);
		roccat_timer_free(priv->timer);
		priv->timer = savu_rmp_timer_to_roccat_timer(timer);
		g_free(timer);
		break;
	default:
		break;
	}

	set_active_iter(key_combo_box);
	set_label(key_combo_box);
}

void savuconfig_key_combo_box_update_rmp(SavuconfigKeyComboBox *key_combo_box, SavuRmp *rmp, guint index) {
	SavuconfigKeyComboBoxPrivate *priv = key_combo_box->priv;
	SavuRmpMacroKeyInfo *macro;
	SavuRmpTimer *timer;

	savu_rmp_set_button(rmp, index, &priv->button);

	switch (priv->button.type) {
	case SAVU_BUTTON_TYPE_MACRO:
		macro = savu_roccat_macro_to_rmp_macro_key_info(priv->macro);
		savu_rmp_set_macro_key_info(rmp, index, macro);
		savu_rmp_macro_key_info_free(macro);
		break;
	case SAVU_BUTTON_TYPE_QUICKLAUNCH:
		savu_rmp_set_launchpath(rmp, index, priv->launchpath);
		break;
	case SAVU_BUTTON_TYPE_TIMER:
		timer = savu_roccat_timer_to_rmp_timer(priv->timer);
		savu_rmp_set_timer(rmp, index, timer);
		g_free(timer);
		break;
	}
}
